model IndoorAirQuality "Demonstrates a room volume with CO2 controls"
  package Medium = Modelica.Media.Air.MoistAir(extraPropertiesNames = {"CO2"}, C_nominal = {1.519E-1}) annotation(Documentation(info = "<html>
<h3>Medium</h3>
    <h4>Description</h4>
    <p>
 A package containing fluid properties.
</p></html>"));
  Modelica.Fluid.Sources.FixedBoundary ambient(nPorts = 1, redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{92, -40}, {72, -20}}, origin = {-2, 0}, rotation = 0), visible = true));
  Modelica.Fluid.Sensors.TraceSubstances CO2conc(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{20, 0}, {40, 20}}, origin = {-10, 0}, rotation = 0), visible = true));
  inner Modelica.Fluid.System system(energyDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, use_eps_Re = true, m_flow_nominal = 0.1, eps_m_flow = 1e-2) annotation(Placement(transformation(extent = {{70, 70}, {90, 90}}, origin = {-10, -150}, rotation = 0), visible = true));
  Modelica.Fluid.Sources.MassFlowSource_T freshAir(redeclare package Medium = Medium, use_m_flow_in = true, nPorts = 2, use_X_in = false, use_C_in = true) annotation(Placement(transformation(extent = {{-70, -40}, {-50, -20}}, origin = {-40, -0}, rotation = 0), visible = true));
  Modelica.Fluid.Vessels.ClosedVolume room(medium(Xi(each nominal = 0.01)), C_start = {1 * massFractionConversion}, redeclare package Medium = Medium, massDynamics = Modelica.Fluid.Types.Dynamics.FixedInitial, use_portsData = false, nPorts = 4, V = volumeOfRoom, X_start = {0.015, 0.085}) annotation(Placement(transformation(extent = {{0, -20}, {20, 0}}, origin = {-10, -0}, rotation = 0), visible = true));
  Modelica.Fluid.Pipes.DynamicPipe airOutlet(mCs_scaled(each nominal = 0.01), mediums(each Xi(each nominal = 0.01)), redeclare package Medium = Medium, diameter = 0.15, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow(show_Res = true), modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b, length = 5) "Outlet duct" annotation(Placement(transformation(extent = {{40, 40}, {60, 20}}, origin = {0, -60}, rotation = 0), visible = true));
  Modelica.Fluid.Sources.MassFlowSource_T source(m_flow = 100 / 1.2 / 3600 * 5, redeclare package Medium = Medium, use_m_flow_in = true, use_C_in = false, C = {100}, nPorts = 2, use_X_in = false) "CO2 emitted by room occupants." annotation(Placement(transformation(extent = {{-38, -98}, {-18, -78}}, origin = {-2, 18}, rotation = 0), visible = true));
  Modelica.Fluid.Pipes.DynamicPipe airInlet(mCs_scaled(each nominal = 0.01), mediums(each Xi(each nominal = 0.01)), redeclare package Medium = Medium, diameter = 0.15, redeclare model FlowModel = Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow(show_Res = true), modelStructure = Modelica.Fluid.Types.ModelStructure.a_v_b, length = 5, nNodes = 1) "Inlet duct" annotation(Placement(transformation(extent = {{-38, 40}, {-18, 20}}, origin = {-2, -60}, rotation = 0), visible = true));
  Modelica.Blocks.Sources.CombiTimeTable people(table = {{0, 0, 0}, {32400, 0, 0}, {32400, 10, 10 * 8.18E-6 / 100}, {39600, 10, 10 * 8.18E-6 / 100}, {39600, 2, 2 * 8.18E-6 / 100}, {46800, 2, 2 * 8.18E-6 / 100}, {46800, 15, 15 * 8.18E-6 / 100}, {54000, 15, 15 * 8.18E-6 / 100}, {54000, 5, 5 * 8.18E-6 / 100}, {64800, 5, 5 * 8.18E-6 / 100}, {64800, 0, 0}, {86400, 0, 0}}) "Time table for number of people and total co2 emitted by people in the room" annotation(Placement(transformation(extent = {{-100, -90}, {-80, -70}}, origin = {20, 10}, rotation = 0), visible = true));
  Modelica.Blocks.Math.Gain massFractionCO2(k = massFractionConversion / 1000) annotation(Placement(visible = true, transformation(origin = {-130, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Fluid.Sensors.TraceSubstances CO2freshAir(redeclare package Medium = Medium) annotation(Placement(transformation(extent = {{20, 0}, {40, 20}}, origin = {-110, 0}, rotation = 0), visible = true));
  parameter Real massFractionConversion = 1.519E-3 "1000 PPM is equal to 1.519E-3 kg/kg";
  parameter Real volumeOfRoom = 100 "Volume of the room";
  parameter Real airExchangeRate = 5 "Air changes per hour";
  parameter Real airDensity = 1.2 "Density of air at standard conditions";
  parameter Real hourToSecs = 3600 "Hours to seconds conversion factor";
  IndoorAirQuality.Controller controller(massFractionConversion = massFractionConversion, volumeOfRoom = volumeOfRoom, airExchangeRate = airExchangeRate, airDensity = airDensity, hourToSecs = hourToSecs, toPPM.k = 1 / (massFractionConversion / 1000)) annotation(Placement(visible = true, transformation(origin = {50, 10}, extent = {{-10, -10}, {10, 10}}, rotation = -720)));

  model Controller "Controller to maintain the carbon dioxide levels."
    Modelica.Blocks.Sources.Constant CO2ref(k = 1000) "Normalized CO2 set point" annotation(Placement(transformation(extent = {{80, 40}, {60, 60}}, origin = {30, -20}, rotation = 0), visible = true));
    Modelica.Blocks.Continuous.LimPID PID(controllerType = Modelica.Blocks.Types.SimpleController.PI, yMax = 0, yMin = -1, Ti = 10, k = 0.01) annotation(Placement(transformation(extent = {{40, 40}, {20, 60}}, origin = {40, -20}, rotation = 0), visible = true));
    Modelica.Blocks.Math.Gain toPPM(k = 1 / massFractionConversion) annotation(Placement(visible = true, transformation(origin = {50, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Blocks.Math.Gain toFlowRate(k = -1 * volumeOfRoom * airDensity / hourToSecs * airExchangeRate) "Nominal fresh air flow rate (for u=1)" annotation(Placement(transformation(extent = {{0, 40}, {20, 60}}, origin = {20, 80}, rotation = -180), visible = true));
    Modelica.Blocks.Interfaces.RealInput u1 annotation(Placement(visible = true, transformation(origin = {-0, -40}, extent = {{-20, -20}, {20, 20}}, rotation = 0), iconTransformation(origin = {-120, 0}, extent = {{-20, -20}, {20, 20}}, rotation = 0)));
    Modelica.Blocks.Interfaces.RealOutput y1 annotation(Placement(visible = true, transformation(origin = {-50, 30}, extent = {{-10, -10}, {10, 10}}, rotation = -180), iconTransformation(origin = {110, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    parameter Real massFractionConversion;
    parameter Real volumeOfRoom;
    parameter Real airExchangeRate;
    parameter Real airDensity;
    parameter Real hourToSecs;
  equation
    connect(CO2ref.y, PID.u_s) annotation(Line(points = {{3.5, 0}, {-3.5, -0}}, color = {1, 37, 163}, visible = true, origin = {85.5, 30}));
    connect(toPPM.y, PID.u_m) annotation(Line(visible = true, origin = {67, -20.667}, points = {{-6, -19.333}, {3, -19.333}, {3, 38.667}}, color = {1, 37, 163}));
    connect(toFlowRate.u, PID.y) annotation(Line(visible = true, origin = {40.5, 30}, points = {{-18.5, 0}, {18.5, 0}}, color = {1, 37, 163}));
    connect(y1, toFlowRate.y) annotation(Line(visible = true, origin = {-25.5, 30}, points = {{-24.5, -0}, {24.5, 0}}, color = {1, 37, 163}));
    connect(u1, toPPM.u) annotation(Line(visible = true, origin = {19, -40}, points = {{-19, 0}, {19, -0}}, color = {1, 37, 163}));
    annotation(Documentation(info = "<html>
<h3>Controller</h3>
    <h4>Description</h4>
    <p>
A model of a PI controller to keep the carbon dioxide level at a reference concentration.
</p></html>"), Diagram(coordinateSystem(extent = {{-75, -70}, {150, 60}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-100, -100}, {100, 100}}), Text(visible = true, textColor = {64, 64, 64}, extent = {{-150, 110}, {150, 150}}, textString = "%name"), Text(visible = true, origin = {10, -7.499}, textColor = {64, 64, 64}, extent = {{-40, -30}, {40, 50}}, textString = "PI")}));
  end Controller;

  Modelica.Blocks.Sources.Constant CO2atm(k = atmosphericCO2) annotation(Placement(visible = true, transformation(origin = {-160, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  parameter Real atmosphericCO2 = 400 "CO2 level in the atmosphere in PPM";
  Modelica.Fluid.Sensors.MassFlowRate massFlowRate(redeclare replaceable package Medium = Medium) annotation(Placement(visible = true, transformation(origin = {-60, -30}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  Modelica.Blocks.Sources.RealExpression CO2fromPeoplePPM(y = source.m_flow_in / massFractionConversion * 1000 * 100) annotation(Placement(visible = true, transformation(origin = {-70, -90}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
equation
  connect(airOutlet.port_b, ambient.ports[1]) annotation(Line(points = {{-5, 0}, {5, 0}}, color = {0, 127, 255}, visible = true, origin = {65, -30}));
  connect(airInlet.port_b, room.ports[1]) annotation(Line(points = {{-11.5, -30}, {7, -30}, {7, -20}, {8.5, -20}}, color = {0, 127, 255}, visible = true, origin = {-8.5, 0}));
  connect(source.ports[1], room.ports[2]) annotation(Line(points = {{-11.5, -70}, {8.5, -70}, {8.5, -20}}, color = {0, 127, 255}, visible = true, origin = {-8.5, 0}));
  connect(room.ports[3], airOutlet.port_a) annotation(Line(points = {{8.5, -20}, {11, -20}, {11, -30}, {48.5, -30}}, color = {0, 127, 255}, visible = true, origin = {-8.5, 0}));
  connect(room.ports[4], CO2conc.port) annotation(Line(points = {{8.5, -20}, {11.147, -20}, {11.147, -26}, {28.5, -26}, {28.5, 0}}, color = {0, 127, 255}, visible = true, origin = {-8.5, 0}));
  connect(people.y[2], source.m_flow_in) annotation(Line(visible = true, origin = {-51.75, -66}, points = {{-7.25, -4}, {-2.25, -4}, {-2.25, 4}, {11.75, 4}}, color = {1, 37, 163}));
  connect(freshAir.C_in[1], massFractionCO2.y) annotation(Line(visible = true, origin = {-114.5, -43.084}, points = {{4.5, 5.084}, {-0.405, 5.084}, {-0.405, 3.084}, {-4.5, 3.084}}, color = {1, 37, 163}));
  connect(CO2conc.C, controller.u1) annotation(Line(visible = true, points = {{-4.5, -0}, {4.5, 0}}, color = {1, 37, 163}, origin = {35.5, 10}));
  connect(freshAir.m_flow_in, controller.y1) annotation(Line(visible = true, points = {{-86, -19.91}, {-106, -19.91}, {-106, 42.09}, {94, 42.09}, {94, 12.09}, {85, 12.09}}, color = {1, 37, 163}, origin = {-24, -2.09}));
  connect(CO2atm.y, massFractionCO2.u) annotation(Line(visible = true, origin = {-145.5, -40}, points = {{-3.5, 0}, {3.5, 0}}, color = {1, 37, 163}));
  connect(freshAir.ports[1], massFlowRate.port_a) annotation(Line(visible = true, origin = {-80, -30}, points = {{-10, 0}, {10, 0}}, color = {0, 127, 255}));
  connect(massFlowRate.port_b, airInlet.port_a) annotation(Line(visible = true, origin = {-45, -30}, points = {{-5, 0}, {5, 0}}, color = {0, 127, 255}));
  connect(freshAir.ports[2], CO2freshAir.port) annotation(Line(visible = true, origin = {-77.5, -15}, points = {{-12.5, -15}, {-2.5, -15}, {-2.5, 15}, {-2.5, 15}}, color = {0, 127, 255}));
  annotation(experiment(StopTime = 86400, __Wolfram_Algorithm = "cvodes", Tolerance = 1e-7), preferredView = "info", uses(Modelica(version = "4.0.0")), __Wolfram(openClass = IndoorAirQuality), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90, -90}, {90, 90}}, radius = 25), Text(visible = true, origin = {12, 20.833}, textColor = {255, 255, 255}, extent = {{-102, -70.833}, {78, 29.167}}, textString = "Ex", fontName = "Arial")}), Diagram(coordinateSystem(extent = {{-180, -104.223}, {100, 50}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})), Documentation(info = "<html>


    <p>
It is often noted that CO2 causes global warming, but keeping track of CO2 concentrations is also an important aspect of preserving indoor air quality. Concentrations of more than 1000 ppm can lead to drowsiness, and long-term exposures can lead to adverse health effects. This example shows how you can control the concentration of CO2 in a room and subsequently check if climate change puts more pressure on the ventilation system.
</p>
    <h4>Effect of Carbon Dioxide Emissions</h4>
    <p>
Atmospheric CO2 level measured at Mauna Loa Observatory in Hawaii peaked near 420 ppm in May 2021. This is an increase of 105 ppm from the 315 ppm that was first recorded at the observatory in 1958. It has resulted in an increase in the required mass flow rate of fresh air by about 16 %. Further increase of CO2 level even by a small margin of 30 ppm to 450 ppm (say by 2050) will lead to an increase in the mass flow rates by about 7%, thus putting additional pressure on the room's ventilation system.
</p><p>
<img src=\"modelica://IndoorAirQuality/DocumentationFiles/mass-flow-rates.png\" alt=\"mass-flow-rates\"/>
</p>


<p>In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at&nbsp;<a href=\"http://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a></p>

<p>For the full example, open the accompanying notebook <a href=\"modelica://IndoorAirQuality/IndoorAirQuality.nb\">IndoorAirQuality.nb</a>.</p>
</html>", figures = {Figure(title = "Mass flow rate", identifier = "d92c5", plots = {Plot(curves = {Curve(y = massFlowRate.m_flow, legend = "mass flow rate of fresh air")}, x = Axis(min = 0, max = 25, unit = "h"))}, caption = "Mass flow rate of fresh air supplied to the room."), Figure(title = "CO2 concentrations", identifier = "480fb", preferred = true, plots = {Plot(curves = {Curve(y = controller.toPPM.y, legend = "CO2 room in ppm")}, x = Axis(unit = "h"), y = Axis(min = 0, max = 1400)), Plot(curves = {Curve(y = CO2fromPeoplePPM.y, legend = "CO2 from people in ppm")}, x = Axis(unit = "h")), Plot(curves = {Curve(y = massFlowRate.m_flow, legend = "mass flow of fresh air")}, x = Axis(unit = "h"))}, caption = "The top plot shows the CO2 level in the room. The middle plot shows the CO2 released from people. The last plot shows the mass flow rate of fresh air fed into the room.")}, revisions = "<html></html>"));
end IndoorAirQuality;
